@@ -13,7 +13,7 @@ PKG_NAME="ukui-screensaver"
 }
 
 which mate-autogen || {
-    echo "You need to install mate-common from the UKUI Git"
+    echo "You need to install mate-common from the MATE Git"
     exit 1
 }
 
