@@ -18,6 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * Authors: William Jon McCann <mccann@jhu.edu>
+ * Modified by: Hao Lee <lihao@kylinos.cn>
  *
  */
 
@@ -39,10 +40,6 @@
 #include <gtk/gtk.h>
 #include <gtk/gtkx.h>
 #include <gio/gio.h>
-#include <gdk/gdk.h>
-
-#include <glib.h>
-#include <glib/gstdio.h>
 
 #define MATE_DESKTOP_USE_UNSTABLE_API
 #include <libmate-desktop/mate-desktop-utils.h>
@@ -826,11 +823,11 @@ frame_pixbuf (GdkPixbuf *source)
 	double           radius;
 	guint8          *data;
 
-	frame_width = 3;
+	frame_width = 3; /* Set border width */
 
 	w = gdk_pixbuf_get_width (source) + frame_width * 2;
 	h = gdk_pixbuf_get_height (source) + frame_width * 2;
-	radius = w / 10;
+	radius = 0; /* Set radius of rounded rectangle */
 
 	dest = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
 						   TRUE,
@@ -852,9 +849,9 @@ frame_pixbuf (GdkPixbuf *source)
 
 	/* set up image */
 	cairo_rectangle (cr, 0, 0, w, h);
-	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 1.0);
+	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.0);
 	cairo_fill (cr);
-	radius = 0; // 去掉圆角
+
 	rounded_rectangle (cr,
 					   1.0,
 					   frame_width + 0.5,
@@ -944,57 +941,13 @@ check_user_file (const gchar *filename,
 	return TRUE;
 }
 
-/* Read avatar file from DBus */
-static GdkPixbuf *get_avatar_pixbuf()
-{
-	int icon_size = 140;
-	GDBusConnection *system_bus = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, NULL);
-	GVariant *res_tuple =
-	g_dbus_connection_call_sync(system_bus,
-				"org.freedesktop.Accounts",
-				"/org/freedesktop/Accounts",
-				"org.freedesktop.Accounts",
-				"FindUserByName",
-				g_variant_new("(s)", g_get_user_name()),
-				NULL, G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL);
-
-	GVariant *res = g_variant_get_child_value(res_tuple, 0);
-	gchar *obj_path = g_strdup(g_variant_get_string(res, NULL));
-	g_variant_unref(res);
-	g_variant_unref(res_tuple);
-	g_print("=================================%s\n",obj_path);
-	res_tuple = g_dbus_connection_call_sync(system_bus,
-				"org.freedesktop.Accounts",
-				obj_path,
-				"org.freedesktop.DBus.Properties",
-				"Get",
-				g_variant_new("(ss)", "org.freedesktop.Accounts.User", "IconFile"),
-				NULL, G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL);
-	g_free (obj_path);
-
-	res = g_variant_get_child_value(res_tuple, 0);
-	GVariant *str_res = g_variant_get_child_value(res, 0);
-	gchar *icon_file = g_strdup(g_variant_get_string(str_res, NULL));
-	g_print("=================================%s\n",icon_file);
-	g_variant_unref(res);
-	g_variant_unref(str_res);
-	g_variant_unref(res_tuple);
-
-	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (icon_file, icon_size, icon_size, NULL);
-
-	if (icon_file)
-		g_free (icon_file);
-
-	return pixbuf;
-}
-
 static gboolean
 set_face_image (GSLockPlug *plug)
 {
 	GdkPixbuf    *pixbuf;
 	const char   *homedir;
 	char         *path;
-	int           icon_size = 160;
+	int           icon_size = 128; /* Set avatar size */
 	gsize         user_max_file = 65536;
 	uid_t         uid;
 
@@ -1006,36 +959,18 @@ set_face_image (GSLockPlug *plug)
 	pixbuf = NULL;
 	if (check_user_file (path, uid, user_max_file, 0, 0))
 	{
-		/*
-		 * If the file .face exist in home directory,
-		 * then use it as avatar.
-		 */
 		pixbuf = gdk_pixbuf_new_from_file_at_size (path,
 		         icon_size,
 		         icon_size,
 		         NULL);
 	}
-	/*
-	 * If .face is not exist, the fallback avatar would be read from DBus.
-	 */
-	if (pixbuf == NULL)
-		pixbuf = get_avatar_pixbuf();
-	/*
-	 * If reading from DBus is failed,
-	 * then using the default_face.png as the fallback avatar.
-	 */
-	if (pixbuf == NULL) {
-		path = "/usr/share/kylin-greeter/default_face.png";
-		if (g_file_test(path, G_FILE_TEST_EXISTS))
-			pixbuf = gdk_pixbuf_new_from_file_at_size(path,
-						icon_size, icon_size, NULL);
-	}
+
 	g_free (path);
 
-
-	/* If pixbuf is still NULL, then return FALSE and the avatar would be blank */
 	if (pixbuf == NULL)
+	{
 		return FALSE;
+	}
 
 	image_set_from_pixbuf (GTK_IMAGE (plug->priv->auth_face_image), pixbuf);
 
@@ -1550,14 +1485,10 @@ gs_lock_plug_enable_prompt (GSLockPlug *plug,
 	gtk_widget_show (plug->priv->auth_unlock_button);
 	gtk_widget_grab_default (plug->priv->auth_unlock_button);
 	gtk_label_set_text (GTK_LABEL (plug->priv->auth_prompt_label), message);
-	gtk_widget_show (plug->priv->auth_prompt_label);
 	gtk_entry_set_visibility (GTK_ENTRY (plug->priv->auth_prompt_entry), visible);
 	gtk_widget_set_sensitive (plug->priv->auth_prompt_entry, TRUE);
 	gtk_widget_show (plug->priv->auth_prompt_entry);
 
-	/* 隐藏 auth_prompt_label */
-	gtk_widget_hide(plug->priv->auth_prompt_label);
-
 	if (! gtk_widget_has_focus (plug->priv->auth_prompt_entry))
 	{
 		gtk_widget_grab_focus (plug->priv->auth_prompt_entry);
@@ -1665,7 +1596,8 @@ get_user_display_name (void)
 
 	name = g_get_real_name ();
 
-	if (strcmp(name, "") == 0 || strcmp (name, "Unknown") == 0)
+	if (name == NULL || g_strcmp0 (name, "") == 0 ||
+	    g_strcmp0 (name, "Unknown") == 0)
 	{
 		name = g_get_user_name ();
 	}
@@ -1888,8 +1820,8 @@ create_page_one (GSLockPlug *plug)
 	                       GTK_ALIGN_CENTER);
 	gtk_widget_set_valign (GTK_WIDGET (vbox),
 	                       GTK_ALIGN_CENTER);
-
 	gtk_notebook_append_page (GTK_NOTEBOOK (plug->priv->notebook), vbox, NULL);
+
 	vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
 
@@ -2082,8 +2014,13 @@ load_theme (GSLockPlug *plug)
 	{
 		GtkCssProvider *style_provider = gtk_css_provider_get_default ();
 		gtk_css_provider_load_from_path (style_provider, css, NULL);
+		GdkScreen *screen = gdk_display_get_default_screen(gdk_display_get_default ());
+		gtk_style_context_add_provider_for_screen (screen,
+						GTK_STYLE_PROVIDER(style_provider),
+						GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 	}
 	g_free (css);
+
 	builder = gtk_builder_new();
 	if (!gtk_builder_add_from_file (builder,gtkbuilder,&error))
 	{
@@ -2094,92 +2031,38 @@ load_theme (GSLockPlug *plug)
 	}
 	g_free (gtkbuilder);
 
-	/*
-	while (!g_file_test("/home/lihao/c", G_FILE_TEST_EXISTS))
-		;
-		*/
-
 	lock_dialog = GTK_WIDGET (gtk_builder_get_object(builder, "lock-dialog"));
 	gtk_container_add (GTK_CONTAINER (plug), lock_dialog);
 
-	/* 获取显示器分辨率大小 */
-	GdkScreen *screen = gdk_screen_get_default();
-	int width = gdk_screen_get_width(screen);
-	int height = gdk_screen_get_height(screen);
-	/* 添加 Layout */
-	GtkWidget *layout = gtk_builder_get_object(builder, "layout");
-	gtk_widget_set_size_request(layout, width, height);
-
-	/* 以登录界面的背景图片作为默认背景 */
-	gchar *picture_filename = "/usr/share/kylin-greeter/background.png";
-	if(!g_file_test(picture_filename, G_FILE_TEST_EXISTS)){
-		/* 如果登录界面的背景图片不存在则读取桌面背景 */
-		GSettings *settings;
-		settings = g_settings_new ("org.mate.background");
-		gchar *picture_filename = g_settings_get_string(
-				settings, "picture-filename");
-		/* 防止前缀出现问题无法解析 */
-		if(g_str_has_prefix(picture_filename, "${prefix}")){
-			gint length = g_strv_length(picture_filename);
-			gchar tmp[50] = "/usr";
-			strcat(tmp, picture_filename+9);
-			g_free(picture_filename);
-			picture_filename = tmp;
-		}
-	}
-	//g_file_set_contents("/home/lihao/桌面/log", picture_filename, -1, NULL);
-
-	/* 读取图片并设置大小 */
-	GtkWidget *image;
-	GdkPixbuf *pixbuf;
-	pixbuf = gdk_pixbuf_new_from_file_at_scale(
-		 picture_filename,
-		 width, height,FALSE , NULL);
-	image = gtk_image_new_from_pixbuf(pixbuf);
-	/* 添加图片层 */
-	gtk_layout_put(GTK_LAYOUT(layout), image, 0, 0);
-	/* 重新添加已经在Glade中添加好的控件，不然所有东西都被图片盖住了 */
-	GtkWidget *hbox1 = gtk_builder_get_object(builder, "hbox1");
-	gtk_layout_put(GTK_LAYOUT(layout), hbox1, width/2 - 270, height/2 - 110);
-	gtk_widget_show(layout);
-
-	/*
-	GtkStyleContext *context = gtk_widget_get_style_context(layout);
-	GtkStateFlags flags = gtk_style_context_get_state(context);
-	GdkRGBA rgb;
-	gtk_style_context_get_background_color(context, flags, &rgb);
-	rgb.red = rgb.green = rgb.blue =0.8;
-	rgb.alpha = 0;
-	gtk_widget_override_background_color(layout, flags, &rgb);
-	*/
-
 	plug->priv->vbox = NULL;
-	//plug->priv->notebook = GTK_WIDGET (gtk_builder_get_object(builder, "notebook"));
+	plug->priv->notebook = GTK_WIDGET (gtk_builder_get_object(builder, "notebook"));
 
 	plug->priv->auth_face_image = GTK_WIDGET (gtk_builder_get_object(builder, "auth-face-image"));
-	//plug->priv->auth_action_area = GTK_WIDGET (gtk_builder_get_object(builder, "auth-action-area"));
-	//plug->priv->auth_time_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-time-label"));
-	//plug->priv->auth_date_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-date-label"));
+	plug->priv->auth_action_area = GTK_WIDGET (gtk_builder_get_object(builder, "auth-action-area"));
+	plug->priv->auth_time_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-time-label"));
+	plug->priv->auth_date_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-date-label"));
 	plug->priv->auth_realname_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-realname-label"));
-	//plug->priv->auth_username_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-username-label"));
+	plug->priv->auth_username_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-username-label"));
 	plug->priv->auth_prompt_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-prompt-label"));
 	plug->priv->auth_prompt_entry = GTK_WIDGET (gtk_builder_get_object(builder, "auth-prompt-entry"));
+	gtk_widget_set_name(plug->priv->auth_prompt_entry, "auth-prompt-entry");
 	plug->priv->auth_prompt_box = GTK_WIDGET (gtk_builder_get_object(builder, "auth-prompt-box"));
-	//plug->priv->auth_capslock_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-capslock-label"));
+	plug->priv->auth_capslock_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-capslock-label"));
 	plug->priv->auth_message_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-status-label"));
 	plug->priv->auth_unlock_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-unlock-button"));
-	//plug->priv->auth_cancel_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-cancel-button"));
-	//plug->priv->auth_logout_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-logout-button"));
-	//plug->priv->auth_switch_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-switch-button"));
-	//plug->priv->auth_note_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-note-button"));
-	//plug->priv->note_tab = GTK_WIDGET (gtk_builder_get_object(builder, "note-tab"));
-	//plug->priv->note_tab_label = GTK_WIDGET (gtk_builder_get_object(builder, "note-tab-label"));
-	//plug->priv->note_ok_button = GTK_WIDGET (gtk_builder_get_object(builder, "note-ok-button"));
-	//plug->priv->note_text_view = GTK_WIDGET (gtk_builder_get_object(builder, "note-text-view"));
-	//plug->priv->note_cancel_button = GTK_WIDGET (gtk_builder_get_object(builder, "note-cancel-button"));
+	gtk_widget_set_name(plug->priv->auth_unlock_button, "auth-unlock-button");
+	plug->priv->auth_cancel_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-cancel-button"));
+	plug->priv->auth_logout_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-logout-button"));
+	plug->priv->auth_switch_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-switch-button"));
+	plug->priv->auth_note_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-note-button"));
+	plug->priv->note_tab = GTK_WIDGET (gtk_builder_get_object(builder, "note-tab"));
+	plug->priv->note_tab_label = GTK_WIDGET (gtk_builder_get_object(builder, "note-tab-label"));
+	plug->priv->note_ok_button = GTK_WIDGET (gtk_builder_get_object(builder, "note-ok-button"));
+	plug->priv->note_text_view = GTK_WIDGET (gtk_builder_get_object(builder, "note-text-view"));
+	plug->priv->note_cancel_button = GTK_WIDGET (gtk_builder_get_object(builder, "note-cancel-button"));
 
 	/* Placeholder for the keyboard indicator */
-	//plug->priv->auth_prompt_kbd_layout_indicator = GTK_WIDGET (gtk_builder_get_object(builder, "auth-prompt-kbd-layout-indicator"));
+	plug->priv->auth_prompt_kbd_layout_indicator = GTK_WIDGET (gtk_builder_get_object(builder, "auth-prompt-kbd-layout-indicator"));
 	if (plug->priv->auth_logout_button != NULL)
 	{
 		gtk_widget_set_no_show_all (plug->priv->auth_logout_button, TRUE);
@@ -2193,42 +2076,36 @@ load_theme (GSLockPlug *plug)
 		gtk_widget_set_no_show_all (plug->priv->auth_note_button, TRUE);
 	}
 
-	char *str;
-	str = "<span color=\"white\" font_desc=\"Ubuntu 23\">%R</span>";
-	gtk_label_set_markup(plug->priv->auth_realname_label, str);
-	str = g_strdup_printf ("<span color=\"white\" "
-			"font_desc=\"Ubuntu 11\">%s</span>", _("Has Logged In"));
-	gtk_label_set_markup(plug->priv->auth_username_label, str);
-	g_free(str);
-	/*
-	GdkRGBA color;
-	gdk_rgba_parse(&color, "#024387");
-	gtk_widget_override_background_color(plug->priv->auth_unlock_button, GTK_STATE_FLAG_NORMAL, &color);
-	gdk_rgba_parse(&color, "#0258AD");
-	gtk_widget_override_background_color(plug->priv->auth_unlock_button, GTK_STATE_FLAG_PRELIGHT, &color);
-	gdk_rgba_parse(&color, "#013C76");
-	gtk_widget_override_background_color(plug->priv->auth_unlock_button, GTK_STATE_FLAG_ACTIVE, &color);
-	gdk_rgba_parse(&color, "#013C76");
-	gtk_widget_override_background_color(plug->priv->auth_unlock_button, GTK_STATE_FLAG_INSENSITIVE, &color);
-	*/
-	/* 按钮字体颜色 */
-	/*
-	GtkWidget *inner_label = gtk_bin_get_child(plug->priv->auth_unlock_button);
-	gdk_rgba_parse(&color, "#ffffff");
-	gtk_widget_override_color(inner_label, GTK_STATE_FLAG_NORMAL, &color);
-	gdk_rgba_parse(&color, "#ffffff");
-	gtk_widget_override_color(inner_label, GTK_STATE_FLAG_PRELIGHT, &color);
-	gdk_rgba_parse(&color, "#ffffff");
-	gtk_widget_override_color(inner_label, GTK_STATE_FLAG_ACTIVE, &color);
-	gdk_rgba_parse(&color, "#ffffff");
-	gtk_widget_override_color(inner_label, GTK_STATE_FLAG_INSENSITIVE, &color);
-	*/
-
 	date_time_update (plug);
 	gtk_widget_show_all (lock_dialog);
 
 	plug->priv->status_message_label = GTK_WIDGET (gtk_builder_get_object(builder, "status-message-label"));
 
+	/* Hide the widget_backup box which contains some widgets we don't use currently */
+	GtkWidget *widget_backup = GTK_WIDGET (gtk_builder_get_object(builder, "widget_backup"));
+	gtk_widget_hide(widget_backup);
+	/* Hide auth_prompt_label which is use to display PAM message */
+	gtk_widget_hide(plug->priv->auth_prompt_label);
+	/* Set widget size of the password entry and the unlock button */
+	gtk_widget_set_size_request(plug->priv->auth_prompt_entry,276,34);
+	gtk_widget_set_size_request(plug->priv->auth_unlock_button,60,37);
+	/*
+	 * From the log information printed in gs_window_move_resize_window(gs-window-x11.c),
+	 * we can learn that under 1920x1080 resolution the initial size of plug is
+	 * 418x200 and its final size is 536x200. This behavior may make gs-window-x11
+	 * calculate the location of lock_box incorrectly, so the lock_box can't be
+	 * centered on the screen. To avoid this situation, fix the size of plug.
+	 */
+	gtk_widget_set_size_request(plug, 536, 200);
+	/* Requisition the auth_username_label to display the login status */
+	char *str;
+	str = g_strdup_printf ("<span color=\"white\" font_desc=\"Ubuntu 11\">%s</span>", _("Has Logged In"));
+	gtk_label_set_markup(plug->priv->auth_username_label, str);
+	g_free(str);
+	/* Set the style of username */
+	str = "<span color=\"white\" font_desc=\"Ubuntu 23\">%R</span>";
+	gtk_label_set_markup(plug->priv->auth_realname_label, str);
+
 	return TRUE;
 }
 
@@ -2390,9 +2267,6 @@ gs_lock_plug_init (GSLockPlug *plug)
 
 	g_signal_connect (plug->priv->auth_unlock_button, "clicked",
 	                  G_CALLBACK (unlock_button_clicked), plug);
-	/* 输入框内敲回车解锁 */
-	g_signal_connect (plug->priv->auth_prompt_entry, "activate",
-	                  G_CALLBACK (unlock_button_clicked), plug);
 
 	g_signal_connect (plug->priv->auth_cancel_button, "clicked",
 	                  G_CALLBACK (cancel_button_clicked), plug);
