@@ -18,6 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * Authors: William Jon McCann <mccann@jhu.edu>
+ * Modified by: Hao Lee <lihao@kylinos.cn>
  *
  */
 
@@ -29,7 +30,6 @@
 #include <string.h>
 
 #include <gdk/gdkx.h>
-#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkx.h>
@@ -43,6 +43,8 @@
 #include <X11/extensions/shape.h>
 #endif
 
+gboolean move_lock_box(gpointer pointer);
+
 static void gs_window_class_init (GSWindowClass *klass);
 static void gs_window_init       (GSWindow      *window);
 static void gs_window_finalize   (GObject       *object);
@@ -64,7 +66,11 @@ enum
 
 struct GSWindowPrivate
 {
+#if GTK_CHECK_VERSION (3, 22, 0)
+	GdkMonitor *monitor;
+#else
 	int        monitor;
+#endif
 
 	GdkRectangle geometry;
 	guint      obscured : 1;
@@ -253,17 +259,6 @@ widget_clear_all_children (GtkWidget *widget)
 	gdk_error_trap_pop_ignored ();
 }
 
-static void
-gs_window_reset_background_surface (GSWindow *window)
-{
-	cairo_pattern_t *pattern;
-	pattern = cairo_pattern_create_for_surface (window->priv->background_surface);
-	gdk_window_set_background_pattern (gtk_widget_get_window (GTK_WIDGET (window)),
-									   pattern);
-	cairo_pattern_destroy (pattern);
-	gtk_widget_queue_draw (GTK_WIDGET (window));
-}
-
 void
 gs_window_set_background_surface (GSWindow        *window,
                                   cairo_surface_t *surface)
@@ -274,57 +269,14 @@ gs_window_set_background_surface (GSWindow        *window,
 	{
 		cairo_surface_destroy (window->priv->background_surface);
 	}
+	window->priv->background_surface = NULL;
 
 	if (surface != NULL)
 	{
 		window->priv->background_surface = cairo_surface_reference (surface);
-		gs_window_reset_background_surface (window);
-	}
-}
-
-static void
-gs_window_clear_to_background_surface (GSWindow *window)
-{
-	g_return_if_fail (GS_IS_WINDOW (window));
-
-	if (! gtk_widget_get_visible (GTK_WIDGET (window)))
-	{
-		return;
 	}
 
-	if (window->priv->background_surface == NULL)
-	{
-		/* don't allow null surfaces */
-		return;
-	}
-
-	gs_debug ("Clearing window to background pixmap");
-
-	gs_window_reset_background_surface (window);
-}
-
-static void
-clear_widget (GtkWidget *widget)
-{
-	GdkRGBA       rgba = { 0.0, 0.0, 0.0, 1.0 };
-	GtkStateFlags state;
-
-	if (!gtk_widget_get_realized (widget))
-	{
-		return;
-	}
-
-	gs_debug ("Clearing widget");
-
-	state = gtk_widget_get_state_flags (widget);
-	gtk_widget_override_background_color (widget, state, &rgba);
-	gdk_window_set_background_rgba (gtk_widget_get_window (widget), &rgba);
-	gtk_widget_queue_draw (GTK_WIDGET (widget));
-
-	/* If a screensaver theme adds child windows we need to clear them too */
-	widget_clear_all_children (widget);
-
-	gdk_flush ();
+	gtk_widget_queue_draw (GTK_WIDGET (window));
 }
 
 void
@@ -332,29 +284,79 @@ gs_window_clear (GSWindow *window)
 {
 	g_return_if_fail (GS_IS_WINDOW (window));
 
-	clear_widget (GTK_WIDGET (window));
-	clear_widget (window->priv->drawing_area);
+	gs_debug ("Clearing widgets");
+
+	if (gtk_widget_get_realized (GTK_WIDGET (window)))
+	{
+		gtk_widget_queue_draw (GTK_WIDGET (window));
+		/* If a screensaver theme adds child windows
+		   we need to clear them too */
+		widget_clear_all_children (GTK_WIDGET (window));
+	}
+
+	if (gtk_widget_get_realized (window->priv->drawing_area))
+	{
+		gtk_widget_queue_draw (window->priv->drawing_area);
+		widget_clear_all_children (window->priv->drawing_area);
+	}
+
+	gdk_flush ();
 }
 
 static cairo_region_t *
 get_outside_region (GSWindow *window)
 {
-	int        i;
+	GdkDisplay *display;
+#if GTK_CHECK_VERSION (3, 22, 0)
+	int         i;
+	int         num_monitors;
+#else
+	GdkScreen  *screen;
+	int         mon;
+#endif
 	cairo_region_t *region;
 
-	region = cairo_region_create ();
-	for (i = 0; i < window->priv->monitor; i++)
-	{
-		GdkRectangle geometry;
-		cairo_rectangle_int_t rectangle;
+	display = gtk_widget_get_display (GTK_WIDGET (window));
+#if !GTK_CHECK_VERSION (3, 22, 0)
+	screen = gdk_display_get_default_screen (display);
+#endif
 
-		gdk_screen_get_monitor_geometry (gtk_widget_get_screen (GTK_WIDGET (window)),
-		                                 i, &geometry);
-		rectangle.x = geometry.x;
-		rectangle.y = geometry.y;
-		rectangle.width = geometry.width;
-		rectangle.height = geometry.height;
-		cairo_region_union_rectangle (region, &rectangle);
+	region = cairo_region_create ();
+
+#if GTK_CHECK_VERSION (3, 22, 0)
+	num_monitors = gdk_display_get_n_monitors (display);
+	for (i = 0; i < num_monitors; i++)
+#else
+	for (mon = 0; mon < window->priv->monitor; mon++)
+#endif
+	{
+#if GTK_CHECK_VERSION (3, 22, 0)
+		GdkMonitor *mon = gdk_display_get_monitor (display, i);
+
+		if (mon != window->priv->monitor)
+		{
+#endif
+			GdkRectangle geometry;
+			cairo_rectangle_int_t rectangle;
+
+#if GTK_CHECK_VERSION (3, 22, 0)
+			gdk_monitor_get_geometry (mon, &geometry);
+#else
+			gdk_screen_get_monitor_geometry (screen, mon,
+			                                 &geometry);
+#endif
+			rectangle.x = geometry.x;
+			rectangle.y = geometry.y;
+			rectangle.width = geometry.width;
+			rectangle.height = geometry.height;
+			cairo_region_union_rectangle (region, &rectangle);
+#if GTK_CHECK_VERSION (3, 22, 0)
+		}
+		else
+		{
+			break;
+		}
+#endif
 	}
 
 	return region;
@@ -369,11 +371,14 @@ update_geometry (GSWindow *window)
 
 	outside_region = get_outside_region (window);
 
+#if GTK_CHECK_VERSION (3, 22, 0)
+	gdk_monitor_get_geometry (window->priv->monitor, &geometry);
+#else
 	gdk_screen_get_monitor_geometry (gtk_widget_get_screen (GTK_WIDGET (window)),
 	                                 window->priv->monitor,
 	                                 &geometry);
-	gs_debug ("got geometry for monitor %d: x=%d y=%d w=%d h=%d",
-	          window->priv->monitor,
+#endif
+	gs_debug ("got geometry for monitor: x=%d y=%d w=%d h=%d",
 	          geometry.x,
 	          geometry.y,
 	          geometry.width,
@@ -385,8 +390,7 @@ update_geometry (GSWindow *window)
 	cairo_region_get_extents (monitor_region, (cairo_rectangle_int_t *)&geometry);
 	cairo_region_destroy (monitor_region);
 
-	gs_debug ("using geometry for monitor %d: x=%d y=%d w=%d h=%d",
-	          window->priv->monitor,
+	gs_debug ("using geometry for monitor: x=%d y=%d w=%d h=%d",
 	          geometry.x,
 	          geometry.y,
 	          geometry.width,
@@ -396,6 +400,21 @@ update_geometry (GSWindow *window)
 	window->priv->geometry.y = geometry.y;
 	window->priv->geometry.width = geometry.width;
 	window->priv->geometry.height = geometry.height;
+
+	/*
+	 * GSWindow uses GtkFixed as its layout, so the child widget can't fill
+	 * the space automatically. We must set the position and the size of
+	 * drawing_area manually after getting the position and the size of
+	 * GSWindow.
+	 * NOTE: The update_geometry function will be called several times, so we
+	 * need to check if the drawing_area has been added to GSWindow already,
+	 * in case it is added redundantly.
+	 */
+	if (!gtk_widget_get_parent(window->priv->drawing_area)) {
+		gtk_widget_set_size_request(window->priv->drawing_area,
+			window->priv->geometry.width, window->priv->geometry.height);
+		gtk_fixed_put(GTK_FIXED(window->priv->vbox),  window->priv->drawing_area, 0, 0);
+	}
 }
 
 static void
@@ -447,6 +466,14 @@ gs_window_move_resize_window (GSWindow *window,
 		                   window->priv->geometry.width,
 		                   window->priv->geometry.height);
 	}
+	/* Print the size of plug area */
+	if (window->priv->lock_box) {
+		GtkRequisition size1, size2;
+		gtk_widget_get_preferred_size(window->priv->lock_box, &size1, &size2);
+		g_print("\n=========================> "
+			"the size of lock_box(lock_socket or plug) is %d %d\n\n",
+			size1.width, size1.height);
+	}
 }
 
 static void
@@ -466,15 +493,15 @@ gs_window_real_unrealize (GtkWidget *widget)
 extern char **environ;
 
 static gchar **
-spawn_make_environment_for_screen (GdkScreen  *screen,
-                                   gchar     **envp)
+spawn_make_environment_for_display (GdkDisplay *display,
+                                    gchar     **envp)
 {
 	gchar **retval = NULL;
-	gchar  *display_name;
+	const gchar *display_name;
 	gint    display_index = -1;
 	gint    i, env_len;
 
-	g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
+	g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
 
 	if (envp == NULL)
 		envp = environ;
@@ -486,7 +513,7 @@ spawn_make_environment_for_screen (GdkScreen  *screen,
 	retval = g_new (char *, env_len + 1);
 	retval[env_len] = NULL;
 
-	display_name = gdk_screen_make_display_name (screen);
+	display_name = gdk_display_get_name (display);
 
 	for (i = 0; i < env_len; i++)
 		if (i == display_index)
@@ -496,18 +523,16 @@ spawn_make_environment_for_screen (GdkScreen  *screen,
 
 	g_assert (i == env_len);
 
-	g_free (display_name);
-
 	return retval;
 }
 
 static gboolean
-spawn_command_line_on_screen_sync (GdkScreen    *screen,
-                                   const gchar  *command_line,
-                                   char        **standard_output,
-                                   char        **standard_error,
-                                   int          *exit_status,
-                                   GError      **error)
+spawn_command_line_on_display_sync (GdkDisplay  *display,
+                                    const gchar  *command_line,
+                                    char        **standard_output,
+                                    char        **standard_error,
+                                    int          *exit_status,
+                                    GError      **error)
 {
 	char     **argv = NULL;
 	char     **envp = NULL;
@@ -520,7 +545,7 @@ spawn_command_line_on_screen_sync (GdkScreen    *screen,
 		return FALSE;
 	}
 
-	envp = spawn_make_environment_for_screen (screen, NULL);
+	envp = spawn_make_environment_for_display (display, NULL);
 
 	retval = g_spawn_sync (NULL,
 	                       argv,
@@ -540,8 +565,9 @@ spawn_command_line_on_screen_sync (GdkScreen    *screen,
 }
 
 static GdkVisual *
-get_best_visual_for_screen (GdkScreen *screen)
+get_best_visual_for_display (GdkDisplay *display)
 {
+	GdkScreen    *screen;
 	char         *command;
 	char         *std_output;
 	int           exit_status;
@@ -552,12 +578,13 @@ get_best_visual_for_screen (GdkScreen *screen)
 	gboolean      res;
 
 	visual = NULL;
+	screen = gdk_display_get_default_screen (display);
 
 	command = g_build_filename (LIBEXECDIR, "ukui-screensaver-gl-helper", NULL);
 
 	error = NULL;
 	std_output = NULL;
-	res = spawn_command_line_on_screen_sync (screen,
+	res = spawn_command_line_on_display_sync (display,
 	        command,
 	        &std_output,
 	        NULL,
@@ -579,8 +606,8 @@ get_best_visual_for_screen (GdkScreen *screen)
 			visual_id = (VisualID) v;
 			visual = gdk_x11_screen_lookup_visual (screen, visual_id);
 
-			gs_debug ("Found best GL visual for screen %d: 0x%x",
-			          gdk_screen_get_number (screen),
+			gs_debug ("Found best GL visual for display %s: 0x%x",
+			          gdk_display_get_name (display),
 			          (unsigned int) visual_id);
 		}
 	}
@@ -598,7 +625,7 @@ widget_set_best_visual (GtkWidget *widget)
 
 	g_return_if_fail (widget != NULL);
 
-	visual = get_best_visual_for_screen (gtk_widget_get_screen (widget));
+	visual = get_best_visual_for_display (gtk_widget_get_display (widget));
 	if (visual != NULL)
 	{
 		gtk_widget_set_visual (widget, visual);
@@ -832,6 +859,27 @@ window_select_shape_events (GSWindow *window)
 #endif
 }
 
+static gboolean
+gs_window_real_draw (GtkWidget *widget,
+                     cairo_t   *cr)
+{
+	GSWindow *window = GS_WINDOW (widget);
+	cairo_surface_t *bg_surface = window->priv->background_surface;
+
+	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+	if (bg_surface != NULL)
+	{
+		cairo_set_source_surface (cr, bg_surface, 0, 0);
+	}
+	else
+	{
+		cairo_set_source_rgb (cr, 0, 0, 0);
+	}
+	cairo_paint (cr);
+
+	return FALSE;
+}
+
 static void
 gs_window_real_show (GtkWidget *widget)
 {
@@ -1082,7 +1130,7 @@ spawn_on_window (GSWindow *window,
 	}
 
 	error = NULL;
-	envp = spawn_make_environment_for_screen (gtk_window_get_screen (GTK_WINDOW (window)), NULL);
+	envp = spawn_make_environment_for_display (gtk_widget_get_display (GTK_WIDGET (window)), NULL);
 	result = g_spawn_async_with_pipes (NULL,
 	                                   argv,
 	                                   envp,
@@ -1151,6 +1199,11 @@ lock_plug_added (GtkWidget *widget,
                  GSWindow  *window)
 {
 	gtk_widget_show (widget);
+	/*
+	 * Move lock_box to the center of screen after 50ms.
+	 * This can reduce the black splash.
+	 */
+	g_timeout_add(50, move_lock_box, window);
 }
 
 static gboolean
@@ -1251,11 +1304,16 @@ create_keyboard_socket (GSWindow *window,
                         guint32   id)
 {
 	int height;
+	int sc_height;
 
-	height = (gdk_screen_get_height (gtk_widget_get_screen (GTK_WIDGET (window)))) / 4;
+	gdk_window_get_geometry (gdk_screen_get_root_window (gtk_widget_get_screen (GTK_WIDGET (window))),
+				 NULL, NULL, NULL, &sc_height);
+
+	height = sc_height / 4;
 
 	window->priv->keyboard_socket = gtk_socket_new ();
-	gtk_widget_set_size_request (window->priv->keyboard_socket, -1, height);
+	gtk_widget_set_size_request(window->priv->keyboard_socket,
+					window->priv->geometry.width, height);
 
 	g_signal_connect (window->priv->keyboard_socket, "destroy",
 	                  G_CALLBACK (keyboard_socket_destroyed), window);
@@ -1263,7 +1321,8 @@ create_keyboard_socket (GSWindow *window,
 	                  G_CALLBACK (keyboard_plug_added), window);
 	g_signal_connect (window->priv->keyboard_socket, "plug_removed",
 	                  G_CALLBACK (keyboard_plug_removed), window);
-	gtk_box_pack_start (GTK_BOX (window->priv->vbox), window->priv->keyboard_socket, FALSE, FALSE, 0);
+	gtk_fixed_put(GTK_FIXED(window->priv->vbox),  window->priv->keyboard_socket,
+				0, window->priv->geometry.height - height);
 	gtk_socket_add_id (GTK_SOCKET (window->priv->keyboard_socket), id);
 }
 
@@ -1420,6 +1479,19 @@ embed_keyboard (GSWindow *window)
 	}
 }
 
+/* Move lock_box to the center of screen */
+gboolean move_lock_box(gpointer pointer)
+{
+	GSWindow *window = pointer;
+	GtkRequisition size1, size2;
+	gtk_widget_get_preferred_size(window->priv->lock_box, &size1, &size2);
+	int x,y;
+	x = (window->priv->geometry.width - size1.width) / 2;
+	y = (window->priv->geometry.height - size1.height) / 2;
+	gtk_fixed_move(window->priv->vbox, window->priv->lock_box, x, y);
+	return FALSE;
+}
+
 static void
 create_lock_socket (GSWindow *window,
                     guint32   id)
@@ -1431,7 +1503,14 @@ create_lock_socket (GSWindow *window,
 	gtk_widget_set_valign (GTK_WIDGET (window->priv->lock_box),
 	                       GTK_ALIGN_CENTER);
 	gtk_widget_show (window->priv->lock_box);
-	gtk_box_pack_start (GTK_BOX (window->priv->vbox), window->priv->lock_box, TRUE, TRUE, 0);
+	/*
+	 * Move lock_box out of the visible range temporarily. It will be moved
+	 * to the center of the screen in lock_plug_added function after the
+	 * plug is plugged onto the socket.
+	 * This will reduce the black splash when cairo draws background
+	 * image on plug.
+	 */
+	gtk_fixed_put(GTK_FIXED(window->priv->vbox),  window->priv->lock_box, -900, -900);
 
 	gtk_container_add (GTK_CONTAINER (window->priv->lock_box), window->priv->lock_socket);
 
@@ -1450,15 +1529,6 @@ create_lock_socket (GSWindow *window,
 	{
 		embed_keyboard (window);
 	}
-	/*
-	GtkStyleContext *context = gtk_widget_get_style_context(window);
-	GtkStateFlags flags = gtk_style_context_get_state(context);
-	GdkRGBA rgb;
-	gtk_style_context_get_background_color(context, flags, &rgb);
-	rgb.red = rgb.green = rgb.blue =0.8;
-	rgb.alpha = 1;
-	gtk_widget_override_background_color(window, flags, &rgb);
-	*/
 }
 
 static void
@@ -1674,12 +1744,6 @@ lock_command_watch (GIOChannel   *source,
 		return FALSE;
 	}
 
-	/*
-	 * gtk_widget_hide is moved from popup_dialog function to here.
-	 * Hide saver after it is covered by lock-dialog, otherwise the
-	 * background switching between window and lock-dialog is too obvious.
-	 */
-	gtk_widget_hide (window->priv->drawing_area);
 	return TRUE;
 }
 
@@ -1752,13 +1816,9 @@ popup_dialog (GSWindow *window)
 		command = g_string_append (command, " --verbose");
 	}
 
-	/*
-	 * Hide drawing_area in function lock_command_watch but not here.
-	 * This can reduce the effect of background switching between window and lock-dialog.
-	 */
-	//gtk_widget_hide (window->priv->drawing_area);
+	gtk_widget_hide (window->priv->drawing_area);
 
-	gs_window_clear_to_background_surface (window);
+	gtk_widget_queue_draw (GTK_WIDGET (window));
 	set_invisible_cursor (gtk_widget_get_window (GTK_WIDGET (window)), FALSE);
 
 	window->priv->dialog_quit_requested = FALSE;
@@ -1848,23 +1908,12 @@ gs_window_set_lock_enabled (GSWindow *window,
 	g_object_notify (G_OBJECT (window), "lock-enabled");
 }
 
-void
-gs_window_set_screen (GSWindow  *window,
-                      GdkScreen *screen)
-{
-
-	g_return_if_fail (GS_IS_WINDOW (window));
-	g_return_if_fail (GDK_IS_SCREEN (screen));
-
-	gtk_window_set_screen (GTK_WINDOW (window), screen);
-}
-
-GdkScreen *
-gs_window_get_screen (GSWindow  *window)
+GdkDisplay *
+gs_window_get_display (GSWindow  *window)
 {
 	g_return_val_if_fail (GS_IS_WINDOW (window), NULL);
 
-	return gtk_widget_get_screen (GTK_WIDGET (window));
+	return gtk_widget_get_display (GTK_WIDGET (window));
 }
 
 void
@@ -1957,8 +2006,12 @@ gs_window_set_status_message (GSWindow   *window,
 }
 
 void
-gs_window_set_monitor (GSWindow *window,
-                       int       monitor)
+gs_window_set_monitor (GSWindow   *window,
+#if GTK_CHECK_VERSION (3, 22, 0)
+                       GdkMonitor *monitor)
+#else
+                       int         monitor)
+#endif
 {
 	g_return_if_fail (GS_IS_WINDOW (window));
 
@@ -1974,10 +2027,18 @@ gs_window_set_monitor (GSWindow *window,
 	g_object_notify (G_OBJECT (window), "monitor");
 }
 
+#if GTK_CHECK_VERSION (3, 22, 0)
+GdkMonitor *
+#else
 int
+#endif
 gs_window_get_monitor (GSWindow *window)
 {
+#if GTK_CHECK_VERSION (3, 22, 0)
+	g_return_val_if_fail (GS_IS_WINDOW (window), NULL);
+#else
 	g_return_val_if_fail (GS_IS_WINDOW (window), -1);
+#endif
 
 	return window->priv->monitor;
 }
@@ -2016,7 +2077,11 @@ gs_window_set_property (GObject            *object,
 		gs_window_set_logout_timeout (self, g_value_get_long (value));
 		break;
 	case PROP_MONITOR:
+#if GTK_CHECK_VERSION (3, 22, 0)
+		gs_window_set_monitor (self, g_value_get_pointer (value));
+#else
 		gs_window_set_monitor (self, g_value_get_int (value));
+#endif
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -2058,7 +2123,11 @@ gs_window_get_property (GObject    *object,
 		g_value_set_long (value, self->priv->logout_timeout);
 		break;
 	case PROP_MONITOR:
+#if GTK_CHECK_VERSION (3, 22, 0)
+		g_value_set_pointer (value, (gpointer) self->priv->monitor);
+#else
 		g_value_set_int (value, self->priv->monitor);
+#endif
 		break;
 	case PROP_OBSCURED:
 		g_value_set_boolean (value, self->priv->obscured);
@@ -2145,16 +2214,23 @@ static gboolean
 gs_window_real_motion_notify_event (GtkWidget      *widget,
                                     GdkEventMotion *event)
 {
-	GSWindow  *window;
-	gdouble    distance;
-	gdouble    min_distance;
-	gdouble    min_percentage = 0.1;
-	GdkScreen *screen;
+	GSWindow   *window;
+	gdouble     distance;
+	gdouble     min_distance;
+	gdouble     min_percentage = 0.1;
+	GdkDisplay *display;
+	GdkScreen  *screen;
+	gint        sc_width;
 
 	window = GS_WINDOW (widget);
 
-	screen = gs_window_get_screen (window);
-	min_distance = gdk_screen_get_width (screen) * min_percentage;
+	display = gs_window_get_display (window);
+	screen = gdk_display_get_default_screen (display);
+
+	gdk_window_get_geometry (gdk_screen_get_root_window (screen), NULL, NULL,
+				 &sc_width, NULL);
+
+	min_distance = sc_width * min_percentage;
 
 	/* if the last position was not set then don't detect motion */
 	if (window->priv->last_x < 0 || window->priv->last_y < 0)
@@ -2354,6 +2430,7 @@ gs_window_class_init (GSWindowClass *klass)
 
 	widget_class->show                = gs_window_real_show;
 	widget_class->hide                = gs_window_real_hide;
+	widget_class->draw                = gs_window_real_draw;
 	widget_class->realize             = gs_window_real_realize;
 	widget_class->unrealize           = gs_window_real_unrealize;
 	widget_class->key_press_event     = gs_window_real_key_press_event;
@@ -2456,13 +2533,20 @@ gs_window_class_init (GSWindowClass *klass)
 
 	g_object_class_install_property (object_class,
 	                                 PROP_MONITOR,
+#if GTK_CHECK_VERSION (3, 22, 0)
+	                                 g_param_spec_pointer ("monitor",
+	                                         "Gdk monitor",
+	                                         "The monitor (in terms of Gdk) which the window is on",
+	                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+#else
 	                                 g_param_spec_int ("monitor",
-	                                         "Xinerama monitor",
-	                                         "The monitor (in terms of Xinerama) which the window is on",
+	                                         "RandR monitor",
+	                                         "The monitor (in terms of RandR) which the window is on",
 	                                         0,
 	                                         G_MAXINT,
 	                                         0,
 	                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+#endif
 
 }
 
@@ -2471,16 +2555,22 @@ create_info_bar (GSWindow *window)
 {
 	window->priv->info_bar = gtk_info_bar_new ();
 	gtk_widget_set_no_show_all (window->priv->info_bar, TRUE);
-	gtk_box_pack_end (GTK_BOX (window->priv->vbox), window->priv->info_bar, FALSE, FALSE, 0);
+	/*
+	 * This widget is used to display the leaving message.
+	 * Since this feature has been removed, we can comment this line directly.
+	 */
+	//gtk_box_pack_end (GTK_BOX (window->priv->vbox), window->priv->info_bar, FALSE, FALSE, 0);
 }
 
-static void
-on_drawing_area_realized (GtkWidget *drawing_area)
+static gboolean
+on_drawing_area_draw (GtkWidget *widget,
+                      cairo_t   *cr)
 {
-	GdkRGBA black = { 0.0, 0.0, 0.0, 1.0 };
+	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+	cairo_set_source_rgb (cr, 0, 0, 0);
+	cairo_paint (cr);
 
-	gdk_window_set_background_rgba (gtk_widget_get_window (drawing_area),
-	                                &black);
+	return FALSE;
 }
 
 static void
@@ -2517,28 +2607,23 @@ gs_window_init (GSWindow *window)
 	                       | GDK_ENTER_NOTIFY_MASK
 	                       | GDK_LEAVE_NOTIFY_MASK);
 
-	window->priv->vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
+	window->priv->vbox = gtk_fixed_new();
 	gtk_widget_show (window->priv->vbox);
 	gtk_container_add (GTK_CONTAINER (window), window->priv->vbox);
 
-	/*
-	GtkStyleContext *context = gtk_widget_get_style_context(window->priv->vbox);
-	GtkStateFlags flags = gtk_style_context_get_state(context);
-	GdkRGBA rgb;
-	gtk_style_context_get_background_color(context, flags, &rgb);
-	rgb.red = rgb.green = rgb.blue =0.8;
-	rgb.alpha = 0;
-	gtk_widget_override_background_color(window->priv->vbox, flags, &rgb);
-	*/
-
 	window->priv->drawing_area = gtk_drawing_area_new ();
 	gtk_widget_show (window->priv->drawing_area);
 	gtk_widget_set_app_paintable (window->priv->drawing_area, TRUE);
-	gtk_box_pack_start (GTK_BOX (window->priv->vbox), window->priv->drawing_area, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (window->priv->vbox),
+	                    window->priv->drawing_area, TRUE, TRUE, 0);
 	g_signal_connect (window->priv->drawing_area,
-	                  "realize",
-	                  G_CALLBACK (on_drawing_area_realized),
+	                  "draw",
+	                  G_CALLBACK (on_drawing_area_draw),
 	                  NULL);
+	/*
+	 * The code which add drawing_area to vbox has been moved into
+	 * the update_geometry function
+	 */
 	create_info_bar (window);
 
 }
@@ -2602,11 +2687,16 @@ gs_window_finalize (GObject *object)
 }
 
 GSWindow *
-gs_window_new (GdkScreen *screen,
-               int        monitor,
+gs_window_new (GdkDisplay *display,
+#if GTK_CHECK_VERSION (3, 22, 0)
+               GdkMonitor *monitor,
+#else
+               int         monitor,
+#endif
                gboolean   lock_enabled)
 {
-	GObject     *result;
+	GObject   *result;
+	GdkScreen *screen = gdk_display_get_default_screen (display);
 
 	result = g_object_new (GS_TYPE_WINDOW,
 	                       "type", GTK_WINDOW_POPUP,
